"""
SBYB API Module.

This module provides the programmatic API for the SBYB library,
allowing users to interact with the library from Python code.
"""

from sbyb.api.interface import SBYB

__all__ = ['SBYB']
