"""
SBYB CLI Module.

This module provides the command-line interface for the SBYB library,
allowing users to interact with the library from the command line.
"""

from sbyb.cli.commands import main

__all__ = ['main']
